# Castorcito
Castorcito allows you to create custom components according to the different needs that your web project may require.
That is why castorcito contains a set of pre-designed components such as Banners, Tabs, Accordions, Quotes, Cards, among others.
To obtain this package of components you must activate the module castorcito_packbase. For a better understanding of the creation of components, you can review the components generated by castorcito_basepack.
If you plan to use one of these components, we recommend you to clone the component and use the clone, this way you will have more control and always keep a backup of the original.

**Note:** This module is currently incompatible with the Klaro Cookie & Consent Manager contrib module. Klaro Cookie & Consent Manager is installed by default in Drupal CMS.

## Managing templates

This module create a formatter to be used in a JSON field.
- First, in managed form display we have to select the option "Castorcito Component" and configurate the component that will be able to this field.
- After that, in managed display we have to select the formatter "Castorcito component" and save that.

This options have to be configurated to templates are able in the site.

The template is defined trought the formatter created by this module. The template is located in the following module's directory:

```
/templates/field--castorcito-component-formatter.html.twig
```

Inside this template there's validations and loops to managed how to render every castorcito field. Every field has a basic html markup inside its related sdc:

```
/components/fields/castorcito_field_name/castorcito_field_name.twig
```

In the twig code, the "component_data" variable has all the information about every component used in this field.

## How to overwrite this markup?

If you want to have a more particular and custom html markup we need to override this template in the theme that you want to see the changes. We recommended to follow the SDC especifications:

```
theme_name/templates/castorcito/field--castorcito-component-formatter.html.twig
```

Inside the template file we have the write the following html markup:

```
{% if component_data is not empty %}
  <div class="c-component c-type--{{ component_data.type|clean_class }}">
  {% if component_data.type|clean_class is not empty%}
    {% embed 'theme_name:' ~ component_data.type|clean_class with {
      component_data: component_data,
    } %}
    {% endembed %}
  {% endif %}
  </div>
{% endif %}
```
**Note:** The embed lines is calling a component defined in the current theme, have to follow the structure.

Every new component should have to related component files (SDC). these have to be created in the following directory path:

```
theme_name/castorcito/components/component_name/component_name.component.yml
theme_name/castorcito/components/component_name/component_name.twig
```

According the single directory component concept, its posible to include in this directory some files related to the styles and behaviour.

```
theme_name/castorcito/components/component_name/component_name.component.yml
theme_name/castorcito/components/component_name/component_name.twig
theme_name/castorcito/components/component_name/component_name.css
theme_name/castorcito/components/component_name/component_name.js
```
# Castorcito_basepack
This castorcito submodule add many default components to be used in multiple scenaries, every one has its own cfields, its configuration and its components files (SDC).
Every component has a default look and feel to be adapted in any current theme, some of them components also include js code to get the properly behaviour.
Be free to test them!


## How to overwrite this components markup?
Sometimes we need to create more custom markup for our solutions. When we're using the components from castorcito_basepack we have to follow the next instructions:

first of all we have to overwrite a template file, so we have to copy the template from the submodule to the current theme:

From the module castorcito:
```
modules/castorcito_basepack/templates/castorcito-basepack--components.html.twig
```

To the current theme:
```
theme_name/templates/castorcito_basepack/castorcito-basepack--components.html.twig
```

We have to create the components files that we want to override for example, we want to overwrite the basepack_text. So, we need to add this files in the current theme:
```
 - components/
 -   basepack_text/
 -     basepack_text.component.yml
 -     basepack_text.twig
```
**Note:** We recommended to use the components inside castorcito_basepack submodule as a guide to have a properly use.

When we know the component we want to overwrite we have to modify the template castorcito-basepack--components.html.twig

Read the twig comments in the following markup to understand whats modifications are made

```
{# Componentes to be overwritten #}
{%
  set overwritten_components = [
    'basepack_text',
  ]
%}

{% set basepacktype = component_data.type|split('_',2)[0] %}
{% if basepacktype == 'basepack' %}
  {# Check if the current component is in the array to be overwritten #}
  {% if component_data.type in overwritten_components %}
    {% if component_data.fields|length > 0 %}
      {# Embed the new component from the current theme #}
      {% embed "theme_name:" ~ component_data.type with {
        component_data: component_data 
      } %}
      {% endembed %}
    {% endif %}
  {% else %}
    {# Here is the normal code from the submodule #}                                                                                                                                                               
    {% if component_data.component_template is not null %}
      {% if component_data.fields|length > 0 %}
        {% embed "castorcito_basepack:" ~ component_data.type with {
          component_data: component_data 
        } %}
        {% endembed %}
      {% endif %}
    {% else  %}
      {% include '@castorcito/field--castorcito-component-formatter.html.twig' with { 'component_data': component_data } %}
    {% endif %}
  {% endif %}

{% else  %}
  {% include '@castorcito/field--castorcito-component-formatter.html.twig' with { 'component_data': component_data } %}
{% endif %}

{{ attach_library('castorcito/castorcito.formatter') }}

```



# Process of upgrading from 1.0.0-alpha1 to latest

- Install config_delete module
- Delete config REST resource block_list_resource
- Upgrade to latest version with composer or git if use dev version
- Go to /update.php

# Installing JSON Editor with composer

One of the dependencies of this module is [json_field], this module adds a solution for adding jsoneditor to a site using Composer.
This solution can be found in the [README.md] of the [json_field] module in the "Installing JSON Editor with composer" section.
