<?php

namespace Drupal\castorcito;

use Drupal\castorcito\Entity\CastorcitoCategory;
use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Url;

/**
 * Provides a listing of Castorcito components.
 */
class CastorcitoComponentListBuilder extends ConfigEntityListBuilder {

  /**
   * {@inheritdoc}
   */
  public function render() {
    $entities = $this->load();
    $categories = [];

    // Organize components by category
    foreach ($entities as $entity) {
      $category = $entity->get('category') ?? 'Uncategorized';
      $categories[$category][] = $entity;
    }

    $build = [
      '#type' => 'container',
      '#attributes' => ['class' => ['castorcito-component-list']],
    ];

    foreach ($categories as $category_name => $entities) {
      $category = CastorcitoCategory::load($category_name);
      $category_label = $category && $category->label() ? $category->label() : $this->t('Unknown Category');
      $header = [
        'label' => $this->t('Label'),
        'id' => $this->t('Machine name'),
        'description' => $this->t('Description'),
        'operations' => $this->t('Operations'),
      ];

      $rows = [];
      foreach ($entities as $entity) {
        $operations = $this->buildOperations($entity);

        $rows[] = [
          'data' => [
            'label' => $entity->label(),
            'id' => $entity->id(),
            'description' => $entity->get('description'),
            'operations' => ['data' => $operations],
          ],
        ];
      }

      $build['category_' . $category_name] = [
        '#type' => 'details',
        '#title' => "{$category_label} (" . count($entities) . ")",
        '#open' => TRUE,
        'table' => [
          '#type' => 'table',
          '#header' => $header,
          '#rows' => $rows,
          '#empty' => $this->t('No components in this category.'),
        ],
      ];
    }

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    $clone = [
      'title' => $this->t('Clone'),
      'weight' => 200,
      'url' => $entity->toUrl('clone-form'),
    ];

    $operations = parent::getDefaultOperations($entity) + [
      'clone' => $clone,
    ];

    // Remove destination URL from the edit link to allow editing component
    // Component.
    if (isset($operations['edit'])) {
      $operations['edit']['url'] = $entity->toUrl('edit-form');
    }

    return $operations;
  }

}
