<?php

namespace Drupal\castorcito\Controller;

use Drupal\castorcito\CastorcitoComponentInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\system\SystemManager;
use Symfony\Component\DependencyInjection\ContainerInterface;



/**
 * Returns responses for CastorcitoController.
 */
class CastorcitoController extends ControllerBase {

  /**
   * System Manager Service.
   *
   * @var \Drupal\system\SystemManager
   */
  protected $systemManager;

  /**
   * The current component.
   *
   * @var \Drupal\castorcito\CastorcitoComponentInterface
   */
  protected $component;


  /**
   * Constructs a new CastorcitoController.
   *
   * @param \Drupal\system\SystemManager $systemManager
   *   System manager service.
   */
  public function __construct(SystemManager $systemManager) {
    $this->systemManager = $systemManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('system.manager')
    );
  }

  /**
   * Renders the admin menu block content.
   *
   * @return array
   *   The render array of the admin menu block.
   */
  public function castorcitoAdminMenuBlockPage() {
    return $this->systemManager->getBlockContents();
  }


  /**
   * Displays the json model or configuration.
   */
  public function viewJson(CastorcitoComponentInterface $castorcito_component) {
    $this->component = $castorcito_component;
    $json = $this->component->get('model');

    return [
      '#theme' => 'castorcito_view_json',
      '#json' => $json,
      '#attached' => [
        'library' => [
          'castorcito/castorcito.admin',
        ],
      ],
    ];
  }

}
